// package main is the main package for the LapisDeploy program
package main

import (
	"encoding/json"
	"fmt"
	"github.com/akamensky/argparse"
	"log"
	"net/http"
	"os"
)

// Create a configuration struct
var configuration = Configuration{}

// fileExists returns whether the given file or directory exists.
func fileExists(path string) (bool, error) {
	_, err := os.Stat(path)
	if err == nil {
		return true, nil
	}
	if os.IsNotExist(err) {
		return false, nil
	}
	return false, err
}

// handler is in charge of handling requests after the JSON has been parsed
func handler(data map[string]interface{}) {
	repository := data["repository"].(map[string]interface{})
	log.Default().Printf("Repo: %s", repository["full_name"])
	sendMessage(MatrixMessage{text: fmt.Sprintf("⚪️ Handling webhook for `%s`...", repository["full_name"])})

	repo_name := repository["name"].(string)

	site, exists, err := getSite(repo_name)
	if err != nil {
		deploy_error := newDeployError(1, "handler", fmt.Sprintf("Failed to check if site '%s' exists", repo_name), fmt.Sprint(err))
		deploy_error.SendOverMatrix()
		return
	}
	if exists {
		sendMessage(MatrixMessage{text: "⚪️ Updating repository..."})
		if deploy_error := site.Update(); deploy_error.code != 0 {
			deploy_error.SendOverMatrix()
			return
		}
		sendMessage(MatrixMessage{text: "⚪️ Restarting server..."})
		if deploy_error := site.Restart(); deploy_error.code != 0 {
			deploy_error.SendOverMatrix()
			return
		}
	} else {
		sendMessage(MatrixMessage{text: "⚪️ Cloning repository..."})
		if deploy_error := CloneSite(repository["ssh_url"].(string), repo_name); deploy_error.code != 0 {
			deploy_error.SendOverMatrix()
			return
		}
		sendMessage(MatrixMessage{text: "⚪️ Starting server..."})
		if site, exists, err = getSite(repo_name); err != nil {
			deploy_error := newDeployError(1, "handler",
				fmt.Sprintf("Failed to get site '%s' after creation!", repo_name), fmt.Sprint(err))
			deploy_error.SendOverMatrix()
		}
		if deploy_error := site.Start(); deploy_error.code != 0 {
			deploy_error.SendOverMatrix()
			return
		}
		defer sendMessage(MatrixMessage{text: "🚀 Launched for the first time!"})
	}
	sendMessage(MatrixMessage{text: "🟢 Deployed successfully!"})
}

// main is the starting point of the program
func main() {
	// Parse arguments
	parser := argparse.NewParser("lapisdeploy", "Easily deploy Lapis web applications through Gitea webhooks")
	config_path := parser.String("c", "config", &argparse.Options{Required: false, Help: "Configuration file", Default: "./config.json"})
	disable_matrix := parser.Flag("M", "disable-matrix", &argparse.Options{Required: false, Help: "Disable Matrix chat bot", Default: false})
	if err := parser.Parse(os.Args); err != nil { // Parse arguments
		fmt.Print(parser.Usage(err)) // Show usage if there's an error
		return
	}

	if exists, err := fileExists(*config_path); !exists {
		fmt.Printf("Invalid config file '%s'\n", *config_path)
		os.Exit(1)
	} else if err != nil {
		panic(err)
	}

	parseConfig(*config_path, &configuration) // Parse JSON configuration file

	http.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) { // Webhook receiver
		var data map[string]interface{}
		err := json.NewDecoder(r.Body).Decode(&data)
		if err != nil {
			log.Panic(err)
			return
		}

		go handler(data) // Run the handler
		fmt.Fprint(w, "Received!")
	})
	log.Printf("Starting Lapis Deploy on port %d...", configuration.port)

	startAllSites()       // Start all the servers
	if !*disable_matrix { // Only start Matrix bot if --disable-matrix isn't passed
		go initMatrix() // Start Matrix stuff in a goroutine
	} else {
		log.Print("Matrix bot is disabled!")
	}
	log.Fatal(http.ListenAndServe(fmt.Sprintf(":%d", configuration.port), nil)) // Start HTTP server
}
